# Program to calculate the  distributions based on the probabilities from rotamer library calculations:
#
# 1. distance distribution between af488 and af568 
# 2. kappa2 dostribution + kappa2 mean value
# 3. FRET distribution for the case of rigid polyproline
#
#	Author: Katarzyna Walczewska-Szewc
#_______________________________________________________________________________________________

from numpy import *
from os import system
from MDAnalysis.coordinates.DCD import *
from MDAnalysis.core.AtomGroup import *
from MDAnalysis.core.Timeseries import *
from MDAnalysis.analysis.align import *
from matplotlib.pyplot import plot, hist, show, figure,title
from random import random

name_out = 'histogram_RL.dat' # name of the output file 

lib1_name = 'AF488_rotamers' #the name of RL dcd and dat files
lib2_name = 'AF594s_rotamers'

dye1_pdb = 'AF488.pdb' #the name of pdb files (if your dcd is without water, the dye should be consistent with is)
dye2_pdb = 'AF594s.pdb'

dye_num1 = 72 #number of dyes atom (only dye, without additional aminoacid)
dye_num2 = 80

asize = 1.5

R0 = 54.

prot_name = 'polyproline.pdb'

# Choose selections for your molecular system 


selection_a1 = "resname CYS and (name CB or name SG or name C or name N or name CA)" #the anchor residue for the first and second dye
selection_a2 = "resname GLY and (name C or name CA or name N)"

selection_d1 = 'resname AX4' # the resname of the first dye
selection_h1 = 'resname AX4 and (name O7 or name C1 or name C10)' #the names of atoms which are in the middle and on the sides of dye head group 

selection_d2 = 'resname AS5' # the resname of the second dye
selection_h2 = 'resname AS5 and (name O7 or name C87 or name C71)'



#Chosing modules to run:

i1 = 1 # Removing the rotamers overlapping with molecular system, generating files with aligned and allowed positions and orientations of dyes 
i2 = 1 # building a histogram of distances, orientation factor and FRET efficiency
i3 = 0 # histograms plots 


#*******************************************************************************************************************
#     Below here, there are no parameters to change, unless you are sure that you want to change them

def val(x):
  v = sqrt(x[0]*x[0] + x[1]*x[1] + x[2]*x[2])
  return v

lib4 = loadtxt(lib1_name+'.dat')
lib5 = loadtxt(lib2_name+'.dat')
  
if i1: #deleting rotamers overlapping with macromolecule + calculate the distances and orientations
  print "deleting rotamers overlapping with macromolecule + aligning to the macromolecule"

  sys2 = Universe(dye2_pdb)
  sys1 = Universe(dye1_pdb)

  #transforming system to the macromolecule coords
  ref = Universe(prot_name)
  
  
  dye1 = sys1.selectAtoms(selection_d1) 
  oo1 = sys1.selectAtoms(selection_h1) 

  dye2 = sys2.selectAtoms(selection_d2) 
  oo2 = sys2.selectAtoms(selection_h2)

  
  
  alignto(sys2,ref,select=selection_a2, mass_weighted=True)
  alignto(sys1,ref,select=selection_a1, mass_weighted=True)


  

 
  pro = ref.selectAtoms('all')
  
  #saving the positions of protein atoms
  savetxt('tmp_polyproline.dat',pro.coordinates())
  #output file
  ff = open("tmp_Distances_orientations.dat",'w')
  
  #Deleting rotamers which overlap with the protein atoms
  name2 = lib2_name+'.dcd'
  sys2.load_new(name2)  # read from a new trajectory file, replace previous trajectory
  name1 = lib1_name+'.dcd'
  sys1.load_new(name1)  # read from a new trajectory file, replace previous trajectory
  dye_positions2 = []
  dye_positions1 = []
  t = 0
  overlaping = 0
 
  for ts in sys2.trajectory[:]: #for AF594 dye library
    

    alignto(sys2,ref,select=selection_a2, mass_weighted=True)
    
    for atom in dye2.coordinates():
      for pratom in pro.coordinates():
	x = atom[0] - pratom[0]
	y = atom[1] - pratom[1]
	z = atom[2] - pratom[2]
	dist = x*x + y*y + z*z 
        if(dist < asize):
	  overlaping = 1
	
    if not overlaping:
      dye_positions2.append(t)
      
    t += 1
    overlaping = 0

  t = 0
  overlaping = 0
  print "finished for AF594 "
  
  for ts in sys1.trajectory[:]:  #for AF488 dye library

    alignto(sys1,ref,select=selection_a1, mass_weighted=True)
    
    for atom in dye1.coordinates():
      for pratom in pro.coordinates():
	x = atom[0] - pratom[0]
	y = atom[1] - pratom[1]
	z = atom[2] - pratom[2]
	dist = x*x + y*y + z*z 
        if(dist < asize)and(atom[2]<17.):
	  overlaping = 1
    if not overlaping:
      dye_positions1.append(t)
      
    t += 1
    overlaping = 0
    
  #Check if the positions of the dye overlap each oher
  
  for i in dye_positions1:
    for j in dye_positions2:
      overlaping = 0
      
      
      for ts in sys1.trajectory[i:i+1]:
	for tt in sys2.trajectory[j:j+1]:
	  alignto(sys1,ref,select=selection_a1, mass_weighted=True) #The dyes positions are aligned to th eprotein again
	  alignto(sys2,ref,select=selection_a2, mass_weighted=True)
	  
	  pos2 = dye2.coordinates()
          pos1 = dye1.coordinates()
      
          for ii in range(dye_num2):
            for jj in range(dye_num1):
	      x = pos2[ii,0] - pos1[jj,0]
	      y = pos2[ii,1] - pos1[jj,1]
	      z = pos2[ii,2] - pos1[jj,2]
	      dist = x*x + y*y + z*z  
	      if(dist < asize):
	        overlaping = 1
   
          if not overlaping :  #calculate the distance and orientation factor for this dye configuration
	
	    o5 = oo2.coordinates()[1,:]
            c15 = oo2.coordinates()[0,:]
            c25 = oo2.coordinates()[2,:]
            o4 = oo1.coordinates()[1,:]
            c14 = oo1.coordinates()[0,:]
            c24 = oo1.coordinates()[2,:]
        
        
            #distance between the middles of dyes head group
            dist = sqrt( (o5[0]-o4[0])**2 + (o5[1]-o4[1])**2 + (o5[2]-o4[2])**2 )
        
        
            #the kappa^2 calculations
            r1 = c15
            r2 = c25

            r3 = c14
            r4 = c24
    
    
   
            r12 = array([r2[0] - r1[0] , r2[1] - r1[1] , r2[2] - r1[2]])
            r12 /= val(r12)
            r43 = array([r3[0] - r4[0] , r3[1] - r4[1] , r3[2] - r4[2]])
            r43 /= val(r43)
            r32 = array([r2[0] - r3[0] , r2[1] - r3[1] , r2[2] - r3[2]])
            r32 /= val(r32)
            r41 = array([r1[0] - r4[0] , r1[1] - r4[1] , r1[2] - r4[2]])
            r41 /= val(r41)
 
    
            p1243 = (r12[0]*r43[0] + r12[1]*r43[1] + r12[2]*r43[2]) 
            p1232 = (r12[0]*r32[0] + r12[1]*r32[1] + r12[2]*r32[2]) 
            p1241 = (r12[0]*r41[0] + r12[1]*r41[1] + r12[2]*r41[2]) 
            p3243 = (r32[0]*r43[0] + r32[1]*r43[1] + r32[2]*r43[2]) 
            p4143 = (r41[0]*r43[0] + r41[1]*r43[1] + r41[2]*r43[2])
    
            kappa2 = (power(p1243 - (3./4.)*(p1232 + p1241)*(p3243 + p4143),2))
            
            #calculations of the FRET efficiency in the static averaging regime 
            eff = 1./(1.+(2./(kappa2*3.))*(power(dist/R0,6)))
            
            #calculation of the weight of this configuration
            waga = lib4[i,-2]*lib5[j,-2]
            
            if not waga==0.0: #calculated parameters are writen to file only if their weight is different than 0
              ff.write( str(waga)+" "+str(dist) + " " + str(kappa2) + " " + str(eff)+"\n")
        
            overlaping = 0

  ff.close()
  print "finished for AF488"

  

  
if i2: #building a histogram of distances, orientation factor and FRET efficiency
  print "building histograms from rotamer data"
  data = loadtxt('tmp_Distances_orientations.dat')

  data[:,0] /=sum(data[:,0]) #renormalisation of the weights
  
  bins = 100
  histogram = zeros((bins, 7)) # 0 dist range, 1 dist.value, 2 kappa^2 range, 3 kappa value, 4 FRET range, 5 FRET_dyn, 6 FRET_stat
  
  #distance
  index = 0
  maxi = max(data[:,1])+1
  mini = min(data[:,1])-1
  
  x =  arange(mini, maxi, (maxi-mini)/float(bins))
  
  histogram[:,0] = x

  for hi in data:
    index = int(((hi[1]-mini)/((maxi-mini)/float(bins))))
    
    histogram[index,1]+=hi[0]


  
  #kappa^2
  index = 0
  maxi = 4.1
  mini = -0.1
  mean_kap = 0.
  
  for hi in data:
    index = int(((hi[2]-mini)/((maxi-mini)/float(bins))))
    histogram[index,3]+=hi[0]
    mean_kap += hi[2]
  histogram[:,2] = arange(mini, maxi, (maxi-mini)/float(bins))
  mean_kap = mean_kap / float(len(data[:,0]))
  print mean_kap
  
  
  #FRET efficiency - static averaging regime
  index = 0
  maxi = 1.05
  mini = -0.05
  
  for hi in data:
    eff = hi[3]
    index = int(((eff-mini)/((maxi-mini)/float(bins))))
    histogram[index,5]+=hi[0]
  histogram[:,4] = arange(mini, maxi, (maxi-mini)/float(bins))
  
  #FRET efficiency - dynamic regime using average value o kappa2

  for hi in data:
    eff = 1./(1.+(2./(mean_kap*3.))*(power(hi[1]/R0,6)))
    
    index = int(((eff-mini)/((maxi-mini)/float(bins))))
    histogram[index,6]+=hi[0]
   
   
  f = open(name_out,'w')
  f.write('#distance  d.probability  kappa^2  k.probability  efficiency  e.probability(static)  e.probability(dynamic) ')
  for line in histogram:
    f.write(str(line[0])+'  '+str(line[1])+'  '+str(line[2])+'  '+str(line[3])+'  '+str(line[4])+'  '+str(line[5])+'  '+str(line[6])+'\n')
  f.close()
  
  
  
  
if i3: #histograms plots
  print "histograms plots"
  
  histogram = loadtxt(name_out)
  
  figure()
  title('distance')
  plot(histogram[:,0],histogram[:,1]/sum(histogram[:,1]))
  figure()
  title('kappa^2')
  plot(histogram[:,2],histogram[:,3]/sum(histogram[:,3]))
  figure()
  title('Efficiency')
  plot(histogram[:,4],histogram[:,5]/sum(histogram[:,5]))
  plot(histogram[:,4],histogram[:,6]/sum(histogram[:,6]),'b.')

  show()
  
  
  
  
  
  
  


  
