#
#  The script extracts information about possible values of dihedral angles
#  from the MD simulation. Each block can be run separately, however some of
#  them require other to be run first.
#
#  Author: Katarzyna Walczewska-Szewc
#_______________________________________________________________________

import os
from numpy import *				# The numpy and matplotlib libraries need to be installed!
from MDAnalysis.coordinates.DCD import *	# as well as MDAnalysis
from MDAnalysis.core.AtomGroup import *
from MDAnalysis.core.Timeseries import *
from matplotlib.pyplot import *

from matplotlib.patches import Ellipse


#Set the name of the output library --> the result will be name_rotamers.dcd and .dat
name = "AF488"
dye_pdb = "AF488.pdb"
dye_dcd = "AF488.dcd"
#set the number of dihedrals in your dye
n_dih = 11

# To run block of program, simply change the if_x value from 0 to 1
if_1 = 1 # extracting dihedrals from MD data, generate a file with indexes of dye atoms
if_2 = 0 # plot histograms to let user to determine the initial values of dihedral angles (need step 1)
if_3 = 0 # creation of the rotamer library (need step 1 and 2)
if_4 = 0 # plot a polar graphs for each dihedral (need step 3)
if_5 = 1 # extract the rotamer frames form MD trajectory (need step 1 and 3)

# Once you are happy about results and you do not want to run separate blocks again, you can remove all tmp_* files
 

#extracting dihedrals from MD data:
if if_1 : #~5 minuts

  #chosing groups of atoms to calculate dihedrals,  need to by typed manually
  x = Universe(dye_pdb)
  
  d0 = x.selectAtoms('bynum 10 or bynum 1 or bynum 2 or bynum 14')
  d1 = x.selectAtoms('bynum 1 or bynum 2 or bynum 14 or bynum 15')
  d2 = x.selectAtoms('bynum 82 or bynum 81 or bynum 78 or bynum 75')
  d3 = x.selectAtoms('bynum 81 or bynum 78 or bynum 75 or bynum 72') 
  d4 = x.selectAtoms('bynum 78 or bynum 75 or bynum 72 or bynum 69')
  d5 = x.selectAtoms('bynum 75 or bynum 72 or bynum 69 or bynum 66')
  d6 = x.selectAtoms('bynum 72 or bynum 69 or bynum 66 or bynum 64')
  d7 = x.selectAtoms('bynum 69 or bynum 66 or bynum 64 or bynum 62')
  d8 = x.selectAtoms('bynum 66 or bynum 64 or bynum 62 or bynum 53')
  d9 = x.selectAtoms('bynum 64 or bynum 62 or bynum 53 or bynum 60')
  d10 =x.selectAtoms('bynum 58 or bynum 49 or bynum 33 or bynum 30')
  
  #This part create a file with indexes of the dye, needed to remove water from rotamer dcd file
  f = open("tmp_dye_atom_list.dat",'w')
  dye_atom_list = ""
  dye = x.selectAtoms('(resname CYS) or (resname AX4)')
  for i in dye.indices():
    dye_atom_list += str(i)+ " "
  f.write(dye_atom_list)
  f.close() 
  
  #define a middle of dye head group and two atoms which determine the orientation
  oo = x.selectAtoms('bynum 39 or bynum 32 or bynum 22') 		

  #output files
  f0 = open("tmp_"+name+"_dih.dat",'w')
  t = 0

  x.load_new(dye_dcd)  # read from a new trajectory file, replace previous trajectory

  for ts in x.trajectory[:]:
    t += 1000
     
    #reading the dihedral values
    dih0 = d0.dihedral()
    dih1 = d1.dihedral()
    dih2 = d2.dihedral()
    dih3 = d3.dihedral()
    dih4 = d4.dihedral()
    dih5 = d5.dihedral()
    dih6 = d6.dihedral()
    dih7 = d7.dihedral()
    dih8 = d8.dihedral()
    dih9 = d9.dihedral()
    dih10 =d10.dihedral()
      
    o = oo.coordinates()
    
    
    
    f0.write( str(t) + " " + str(dih0) + " " + str(dih1)+ " " + str(dih2)+" "+ str(dih3) + " " + str(dih4)+ " " + str(dih5)+ " " +str(dih6)+" " + str(dih7) + " " + str(dih8)+\
                         " " + str(dih9) + " " + str(dih10)+" " + str(o[0,0]) + " " + str(o[0,1]) + " " +str(o[0,2]) + " " +\
                  str(o[1,0]) + " " + str(o[1,1]) + " " +str(o[1,2]) + " " +str(o[2,0]) + " " +str(o[2,1]) + " " +str(o[2,2]) +"\n")

  f0.close()
  

  
#ploting histograms
if if_2 :
  hi0 = loadtxt("tmp_"+name+'_dih.dat')
 
  figure(figsize=(20,10))
  
  for i in [0,1,2,3,4,5]: # Here, only first 6 dihedrals are plotted
    nr = 10*34 + i+1
  
  #for i in [6,7,8,9,10]: # to drow the next 5, use this command
    #nr = 10*34 + i-5
   
    subplot(nr)

    hist(hi0[:,i+1]+180., bins = 360)
    xlabel('dihedral'+str(i))
    xlim(0,360)

  show()

 
 
#the number of values possible for each dihedral, need to by typed manually
n_dih_0 = 2
n_dih_1 = 2
n_dih_2 = 2
n_dih_3 = 3
n_dih_4 = 3
n_dih_5 = 3
n_dih_6 = 3
n_dih_7 = 3
n_dih_8 = 1
n_dih_9 = 4
n_dih_10 = 2
 
number_of_values = [n_dih_0, n_dih_1, n_dih_2, n_dih_3, n_dih_4, n_dih_5, n_dih_6, n_dih_7, n_dih_8, n_dih_9, n_dih_10]
  
#dihedrals values, for each dihedral the proper number of values should be puted here,  need to by typed manually
dih = array([68.,175.,  180.,357.,  95.,264.,  0.,114.,243.,  0.,109.,249.,  1.,107.,249.,  115.,245.,358.,  0.,85.,271.,  12.,  49.,123.,251.,317.,  245.,291.] )
 
# Creation of the rotamer library
if if_3 :
  
  hi0 = loadtxt("tmp_"+name+'_dih.dat')
  length = len(hi0[:,0])
  #creation of output array. The first column is the number of frame, then n_dih dihedral values, and x y z coordinates of the middle of the headgroup
  hi_out = zeros((length, n_dih+5))
  hi_out[:,0] = hi0[:,0]
  for di in range(1,n_dih+1):
    hi_out[:,di] = hi0[:,di]+180. 
  for co in [n_dih + 2, n_dih + 3, n_dih + 4]:
    hi_out[:,co] = hi0[:,co-1]

  
  
  groups = 1.
  for n in number_of_values:
    groups *=n
  groups = int(groups)
  print " quantity of rotamers: ", groups, "\n"
  
  
  #Define all possible combination of dihedral values and assign a number to describe them
  rotamers = zeros((groups,n_dih+1))
  print " Canonical rotamers:\n"

    
    
  r = 0
  
  for i0 in [0,1]:  #Unfortunately numbers here need to be writen manually. The 0 dihedral has two values, so [0,1], first dihedral-two values [2,3] ... nineth dihadrals- for values [22,23,24,25] and so on... 
    for i1 in [2,3]:
      for i2 in [4,5]:
        for i3 in [6,7,8]:
	  for i4 in [9,10,11]:
	    for i5 in [12,13,14]:
	      for i6 in [15,16,17]:
		for i7 in [18,19,20]:
		  for i8 in [21]:
		    for i9 in [22,23,24,25]:
		      for i10 in [26,27]:
		      
		      
			 
                        rotamers[r,0] = r 
		        rotamers[r,1] = dih[i0]
		        rotamers[r,2] = dih[i1]
		        rotamers[r,3] = dih[i2]
	     	        rotamers[r,4] = dih[i3]
		        rotamers[r,5] = dih[i4]
		        rotamers[r,6] = dih[i5]
		        rotamers[r,7] = dih[i6]
		        rotamers[r,8] = dih[i7]
		        rotamers[r,9] = dih[i8]
		        rotamers[r,10]= dih[i9]
		        rotamers[r,11]= dih[i10]
		        r += 1
	  

  print "The array of possible dihedral values combinations - rotamers:"
  for rr in range(groups):  
    print rr, rotamers[rr,:]

  
  hi_canonical = zeros((length,n_dih+1)) #Here the infromation about rotamers is stored
  hi_canonical[:,0] = hi0[:,0] 
  

  #To each frame from MD simulation, one of the rotamers is assigned using last square method.
  
  a = zeros((11,1))
  a += 5000000.
  for i in range(length):
    
    dih_index = 0
    tmpp = 0
    for ii, di in zip(range(1,n_dih+1),number_of_values):
    
      for dd in range(di):
        a[dd] = min([(hi_out[i,ii] - dih[dih_index])**2 , ((hi_out[i,ii]-360.) - dih[dih_index])**2 , ((hi_out[i,ii]+360.) - dih[dih_index])**2])
        dih_index += 1
    
    
      jj = 0
      for m in a[:di]:
        if m==min(a[:di]):
	  hi_canonical[i,ii] = dih[tmpp + jj]
	  jj = 0
        jj += 1
      tmpp += di
   
    #the rotamer identifier is assigned to each frame here
    for konf in range(groups):
      aa = True 
      for di in range(n_dih):
	aa = aa and (hi_canonical[i,di+1] == rotamers[konf,di+1])
	
      if aa: 
        hi_out[i,n_dih+1] = rotamers[konf,0]
    
  savetxt("tmp_DIH_conf_"+name+".dat", hi_out) # Here is a temporary file with MD frames with the dihedral numbers assigned
  
  #All MD frames with the same rotamer assigned are gathered together and the value of each dihedral angle is averaged
  print "Mean dihedrals calculation for each rotamer:\n"
  sum_dih = zeros((groups,n_dih,2)) 
  dindex = zeros((groups,1))
  for i in range(length):
    
    for rot in range(groups):
      
      if hi_out[i,n_dih+1] == float(rot):
	for di in range(n_dih):
	  val = hi_out[i,di+1]*2.*pi/360.
	  sum_dih[rot, di,1] += sin(val)
	  sum_dih[rot, di,0] += cos(val)
	dindex[rot] += 1
  for rot in range(groups):
    sum_dih[rot, :, 0] /= dindex[rot]
    sum_dih[rot, :, 1] /= dindex[rot]
    tmppp = arctan2(sum_dih[rot, :, 1],sum_dih[rot, :, 0])*(360./(2.*pi)) 
    sum_dih[rot, :, 0] = tmppp
    for item in sum_dih[rot, :, 0]:
      if item < 0. :
        item = 360. + item

    print sum_dih[rot,:,0]
    
  #The MD frame is analysed again, but this time instead of using dihedrals values given by user, the average values (different in each rotamer) are used
  aa = zeros((n_dih,3))
  aa += 500000.
  for i in range(length):
    
    dind = 0
    asum = 10000.
    rind = 0
    for rot in sum_dih[:,:,0]:
      
    
      aa[:,0] = (hi_out[i,1:n_dih+1] - rot[:])**2
      aa[:,1] = ((hi_out[i,1:n_dih+1]-360.) - rot[:])**2
      aa[:,2] = ((hi_out[i,1:n_dih+1]+360.) - rot[:])**2
      
      for a in range(len(aa[:,0])):
	aa[a,0] = min(aa[a,:])
      if asum > sum(aa[:,0]):
	asum = sum(aa[:,0])
	dind = rind
      rind += 1
    hi_out[i,n_dih+1] = dind
    

  print "Second calculation of mean dihedrals for each rotamer:\n"
  sum_dih = zeros((groups,n_dih,2))
  dindex = zeros((groups,1))
  for i in range(length):
    
    for rot in range(groups):
      
      if hi_out[i,n_dih+1] == float(rot):
	for di in range(n_dih):
	  val = hi_out[i,di+1]*2.*pi/360.
	  sum_dih[rot, di,1] += sin(val)
	  sum_dih[rot, di,0] += cos(val)
	dindex[rot] += 1
  for rot in range(groups):
    sum_dih[rot, :, 0] /= dindex[rot]
    sum_dih[rot, :, 1] /= dindex[rot]
    tmppp = arctan2(sum_dih[rot, :, 1],sum_dih[rot, :, 0])*(360./(2.*pi)) 
    sum_dih[rot, :, 0] = tmppp
    for item in sum_dih[rot, :, 0]:
      if item < 0. :
        item = 360. + item

    print sum_dih[rot,:,0]
 
 
 
 
    
    
  #Here the representive confromation of dye from real MD trajectory is chosen for each rotamer. (The last square method)
  min_index = zeros((groups,2)) #col 0 : index of representive frame, col 1: standard deviation
  min_index[:,1] = 1000000
  for i in range(length):
    gr = int(hi_out[i,n_dih + 1])
    tmp = 0.
    for di in range(n_dih):
      tmp += min([power(hi_out[i,di+1] - sum_dih[ gr , di ,0],2), power(hi_out[i,di+1] + 360. - sum_dih[ gr , di,0 ],2), power(hi_out[i,di+1] -360. - sum_dih[ gr , di,0 ],2)])
    
    if tmp <  min_index[gr,1]:
      min_index[gr,1] = tmp
      min_index[gr,0] = i
  
      

  
    print "Representative real rotamer for each group:\n"
  out = zeros((groups,n_dih+2))
 
  for rot in range(groups):
    for di in range(n_dih):
      
      sum_dih[rot, di,0] = hi_out[int(min_index[rot,0]), di+1]
  
      
    print sum_dih[rot,:,0] ,"weight of rotamer: ", dindex[rot]/sum(dindex)
    out[rot,0:n_dih] = sum_dih[rot,:,0]
    out[rot,n_dih] = dindex[rot]/sum(dindex)
    out[rot,n_dih+1] =  int(min_index[rot,0])
  savetxt(name+'_rotamers.dat',out) #Here is all neccessary information needed to build a dcd file of rotamer library from MD trajectory.
  
  

if if_4:  #the polar plots of dihedrals value
    
  hi_out = loadtxt("tmp_DIH_conf_"+name+".dat")
  lib = loadtxt(name+'_rotamers.dat')
  length = len(hi_out[:,0])
  y = zeros((360,n_dih+1))
  y[:,0] = arange(0.,360.,1.)

  for i in range(length):
    for j in range(n_dih):
      x = hi_out[i,j+1]
      if x < 0 :
	x+=360.
      index2 = int((x) + 0.5)
      y[index2-1,j+1] += 1
  
  
    
    

    
  fig = figure(figsize=(20,20))
  n_min = number_of_values[1]
  for di,nr in zip(range(1,n_dih),number_of_values[1:]):
    
    numer = 340 + di
    if di==10:
      numer = 340
   
  #plot(y[:,0], y[:,1]) 
    n_max = n_min + nr
  
    ax = fig.add_subplot(numer, polar=True)
    ax.set_yticklabels(' ')
    ax.plot(y[:,0]*2.*pi/360., y[:,di+1]/max(y[:,di+1]), lw=2) #MD out[rot,0:n_dih] = sum_dih[rot,:,0]out[rot,n_dih] = dindex[rot]/sum(dindex)
    ax.vlines(dih[n_min:n_max]*2.*pi/360., 0,1, lw=2,colors='green')
    ylim(0.,1.)
  #ax.plot(array([10,10,38])*2.*pi/360., [1,0,1], lw=2) #canonical
  #Rotamer dihedrals
    ax.vlines(lib[:,di]*2.*pi/360., 0 , (lib[:,n_dih])*n_dih, lw=2, colors='red')
    n_min += nr 
    

  
  show()
  

if if_5: # Generate a dcd file containing only rotamer frames - need the catdcd program to be installed

  print "generation of dcd file containing only rotamer trajectories"
  
  lib = loadtxt(name+'_rotamers.dat') #the information about rotamer library is loaded
  
  index = lib[:,-1] #in last column I have information about frame with representive configuration. All rotamers with the non zero weight are considered
  command = ''
  for i in index[:]:
    if i > 0.:
      
      os.system('catdcd -o tmp_dcd_'+str(int(i))+'.dcd -i tmp_dye_atom_list.dat -first '+str(int(i)+1)+' -last '+str(int(i)+1)+ ' ' + name + '.dcd')

      command += ' tmp_dcd_'+str(int(i))+'.dcd'
  os.system('catdcd -o '+name+'_rotamers.dcd'+ command)
  os.system('rm' + command)
  
print "The rotamer library calculation fo the dye ",name, " is finished" 
  
  
  
  
  
  
  